#!/bin/sh

log_debug()  {
	if [ "${SETTINGS_DEBUG_LOGGING}" = "1" ]; then
		logger -t "ffac-ssid-changer" -p debug "$@"
	fi
}

safety_exit() {
	log_debug "$1, exiting with error code 2"
	exit 2
}
SETTINGS_DEBUG_LOGGING="$(uci -q get ssid-changer.settings.debug_log_enabled)"
: "${SETTINGS_DEBUG_LOGGING:="0"}"

pgrep -f autoupdater >/dev/null && safety_exit 'autoupdater running'
UT=$(sed 's/\..*//g' /proc/uptime)
[ "$UT" -gt 60 ] || safety_exit 'less than one minute'
[ "$(find /var/run -name "hostapd-*" | wc -l)" -gt 0 ] || safety_exit 'no hostapd-*'

MINUTES="$(uci -q get ssid-changer.settings.switch_timeframe)"
: "${MINUTES:=30}"

FIRST="$(uci -q get ssid-changer.settings.first)"
: "${FIRST:=5}"


PREFIX="$(uci -q get ssid-changer.settings.prefix)"
: "${PREFIX:=FF_Offline_}"

PREFIX_OWE="$(uci -q get ssid-changer.settings.prefix_owe)"
: "${PREFIX_OWE:=FF_Off_OWE}"

if [ "$(uci -q get ssid-changer.settings.enabled)" = '0' ]; then
	DISABLED='1'
else
	DISABLED='0'
fi

SETTINGS_SUFFIX="$(uci -q get ssid-changer.settings.suffix)"
: "${SETTINGS_SUFFIX:=nodename}"

if [ "$SETTINGS_SUFFIX" = 'nodename' ]; then
	SUFFIX="$(uname -n)"
	if [ ${#SUFFIX} -gt $((30 - ${#PREFIX})) ]; then
		max_suffix_length=$(( (28 - ${#PREFIX} ) / 2 ))
		suffix_first_chars="$(printf '%s' "$SUFFIX" | head -c ${max_suffix_length})"
		suffix_last_chars="$(printf '%s' "$SUFFIX" | tail -c ${max_suffix_length})"
		SUFFIX="${suffix_first_chars}...${suffix_last_chars}"
	fi
elif [ "$SETTINGS_SUFFIX" = 'mac' ]; then
	SUFFIX="$(uci -q get network.bat0.macaddr | /bin/sed 's/://g')"
else
	SUFFIX=''
fi

OFFLINE_SSID="$PREFIX$SUFFIX"
OFFLINE_SSID_OWE="$PREFIX_OWE$SUFFIX"

ONLINE_SSID_OWE="$(uci -q get wireless.owe_radio0.ssid)"
[ -n "$ONLINE_SSID_OWE" ] && OWE=true || OWE=false

ONLINE_SSIDs="$(uci show | grep "wireless.client_radio[0-9]\." | grep ssid  | awk -F '='  '{print $2}' | sed "s/\\\'/TICX/g" | tr \' \~ | sed "s/TICX/\\\'/g" ) "
: "${ONLINE_SSIDs:=~FREIFUNK~}"

TMP=/tmp/ssid-changer-count
if [ ! -f $TMP ]; then echo "0">$TMP; fi
OFF_COUNT=$(cat $TMP)

TQ_LIMIT_ENABLED="$(uci -q get ssid-changer.settings.tq_limit_enabled)"
: "${TQ_LIMIT_ENABLED:=0}"

if [ "$TQ_LIMIT_ENABLED" = 1 ]; then
	TQ_LIMIT_MAX="$(uci -q get ssid-changer.settings.tq_limit_max)"
	: "${TQ_LIMIT_MAX:=45}"
	TQ_LIMIT_MIN="$(uci -q get ssid-changer.settings.tq_limit_min)"
	: "${TQ_LIMIT_MIN:=35}"
	GATEWAY_TQ=0
	if [ -f /var/gluon/state/has_default_gw4 ]; then
		GATEWAY_TQ="$(batctl gwl -H | grep -e "^\*" | awk -F'[()]' '{print $2}' | tr -d " ")"
	fi

	MSG="TQ is $GATEWAY_TQ, "

	if [ "$GATEWAY_TQ" -ge "$TQ_LIMIT_MAX" ]; then
		CHECK=1
	elif [ "$GATEWAY_TQ" -lt "$TQ_LIMIT_MIN" ]; then
		CHECK=0
	else
		log_debug "TQ is $GATEWAY_TQ, do nothing"
		exit 0
	fi
else
	MSG=""
	if [ -f /var/gluon/state/has_default_gw4 ]; then
		CHECK=1
	else
		CHECK=0
	fi
fi

UP=$((UT / 60))
M=$((UP % MINUTES))

HUP_NEEDED=0
if [ "$CHECK" -gt 0 ] || [ "$DISABLED" = '1' ]; then
	log_debug "node is online"
	LOOP=1
	for HOSTAPD in /var/run/hostapd-*.conf; do
		[ -e "$HOSTAPD" ] || break  # handle the case of no hostapd-* files
		grep "^bridge=br-client" "$HOSTAPD" > /dev/null || continue # handle case of private wifi
		ONLINE_SSID="$(echo $ONLINE_SSIDs | awk -F '~' -v l=$((LOOP*2)) '{print $l}')"
		LOOP=$((LOOP+1))
		CURRENT_SSID="$(grep "^ssid=$ONLINE_SSID" "$HOSTAPD" | cut -d"=" -f2)"
		if [ "$CURRENT_SSID" = "$ONLINE_SSID" ]; then
			log_debug "SSID $CURRENT_SSID is correct, nothing to do"
			continue
		fi
		CURRENT_SSID="$(grep "^ssid=$OFFLINE_SSID" "$HOSTAPD" | cut -d"=" -f2)"
		if [ "$CURRENT_SSID" = "$OFFLINE_SSID" ]; then
			logger -s -t "ffac-ssid-changer" -p 5 "$MSG""SSID is $CURRENT_SSID, change to $ONLINE_SSID"
			sed -i "s~^ssid=$CURRENT_SSID~ssid=$ONLINE_SSID~" "$HOSTAPD"
			HUP_NEEDED=1
		else
			logger -s -t "ffac-ssid-changer" -p 5 "could not set to online state: did neither find SSID '$ONLINE_SSID' nor '$OFFLINE_SSID'. Please reboot"
		fi
		if [ "$OWE" = true ]; then
			CURRENT_SSID_OWE="$(grep "^ssid=$OFFLINE_SSID_OWE" "$HOSTAPD" | cut -d"=" -f2)"
			if [ "$CURRENT_SSID_OWE" = "$OFFLINE_SSID_OWE" ]; then
				logger -s -t "ffac-ssid-changer" -p 5 "$MSG""OWE SSID is $CURRENT_SSID_OWE, change to $ONLINE_SSID_OWE"
				sed -i "s~^ssid=$CURRENT_SSID_OWE~ssid=$ONLINE_SSID_OWE~" "$HOSTAPD"
				HUP_NEEDED=1
			else
				logger -s -t "ffac-ssid-changer" -p 5 "could not set to online state: did neither find OWE SSID '$ONLINE_SSID_OWE' nor '$OFFLINE_SSID_OWE'. Please reboot"
			fi
		fi
	done
elif [ "$CHECK" -eq 0 ]; then
	log_debug "node is considered offline"
	if [ $UP -lt "$FIRST" ] || [ $M -eq 0 ]; then
		if [ $UP -lt "$FIRST" ]; then
			T=$FIRST
		else
			T=$MINUTES
		fi
		if [ "$OFF_COUNT" -ge $((T / 2)) ]; then
			LOOP=1
			for HOSTAPD in /var/run/hostapd-*.conf; do
				[ -e "$HOSTAPD" ] || break  # handle the case of no hostapd-* files
				grep "^bridge=br-client" "$HOSTAPD" || continue # handle case of private wifi
				ONLINE_SSID="$(echo $ONLINE_SSIDs | awk -F '~' -v l=$((LOOP*2)) '{print $l}')"
				LOOP=$((LOOP+1))
				CURRENT_SSID="$(grep "^ssid=$OFFLINE_SSID" "$HOSTAPD" | cut -d"=" -f2)"
				if [ "$CURRENT_SSID" = "$OFFLINE_SSID" ]; then
					log_debug "SSID $CURRENT_SSID is correct, nothing to do"
					continue
				fi
				CURRENT_SSID="$(grep "^ssid=$ONLINE_SSID" "$HOSTAPD" | cut -d"=" -f2)"
				if [ "$CURRENT_SSID" = "$ONLINE_SSID" ]; then
					logger -s -t "ffac-ssid-changer" -p 5 "$MSG""$OFF_COUNT times offline, SSID is $CURRENT_SSID, change to $OFFLINE_SSID"
					sed -i "s~^ssid=$ONLINE_SSID~ssid=$OFFLINE_SSID~" "$HOSTAPD"
					HUP_NEEDED=1
				else
					logger -s -t "ffac-ssid-changer" -p 5 "could not set to offline state: did neither find SSID '$ONLINE_SSID' nor '$OFFLINE_SSID'. Please reboot"
				fi
				if [ "$OWE" = true ]; then
					CURRENT_SSID_OWE="$(grep "^ssid=$ONLINE_SSID_OWE" "$HOSTAPD" | cut -d"=" -f2)"
					if [ "$CURRENT_SSID_OWE" = "$ONLINE_SSID_OWE" ]; then
						logger -s -t "ffac-ssid-changer" -p 5 "$MSG""$OFF_COUNT times offline, SSID is $CURRENT_SSID_OWE, change to $OFFLINE_SSID_OWE"
						sed -i "s~^ssid=$ONLINE_SSID_OWE~ssid=$OFFLINE_SSID_OWE~" "$HOSTAPD"
						HUP_NEEDED=1
					else
						logger -s -t "ffac-ssid-changer" -p 5 "could not set to offline state: did neither find SSID '$ONLINE_SSID_OWE' nor '$OFFLINE_SSID_OWE'. Please reboot"
					fi
				fi
			done
		fi
	fi
	echo "$((OFF_COUNT + 1))">$TMP
fi

if [ $HUP_NEEDED = 1 ]; then
	ubus call hostapd reload
	if [ -f /lib/gluon/eulenfunk-hotfix/check_hostapd.sh ] ; then
	   sleep 2 # settle down
	   ps|grep hostapd|grep .pid|xargs -n 10 /lib/gluon/eulenfunk-hotfix/check_hostapd.sh
	fi
	HUP_NEEDED=0
	log_debug "Sent HUP to all hostapd to load the new SSID"
fi

if [ $M -eq 0 ]; then
	echo "0">$TMP
fi
